import numpy as np
import laspy
from itertools import groupby
from process_lidar_data import calculate_corners, define_gridareas

# hard coded files for test data
lazData_path = ["server/example_lidar_data/ot_N_000005_1.laz", "server/example_lidar_data/ot_N_000033_1.laz"]

# Info about data
with laspy.open(lazData_path[0]) as fh:
    # Print metadata properties
    print("File Version:", fh.header.version)
    print("Point Count:", fh.header.point_count)
    print("Scale Factors:", fh.header.scale)
    print("Offset:", fh.header.offset)

    print('Points from Header:', fh.header.point_count)
    las = fh.read()
    print(las)
    print('Points from data:', len(las.points))
    ground_pts = las.classification == 2
    bins, counts = np.unique(las.return_number[ground_pts], return_counts=True)
    print('Ground Point Return Number distribution:')
    for r, c in zip(bins, counts):
        print('    {}:{}'.format(r, c))

# check if lidar points is within range of the area selected
def inArea(position, areaRange):
    x, y, _ = position
    if (areaRange[0][0] < x < areaRange[1][0]) and (areaRange[0][1] > y > areaRange[1][1]):
        return True
    else:
        return False

# Calculation of height in an area from las files
def find_height(points):
    height_differences = [] # final container for height data

    # sort the points
    sorted_coords = sorted(points, key=lambda coord: (coord[0],coord[1]))

    # group the sorted points that has the same xy- coordinates together
    groupCoords = [list(group) for key, group in groupby(sorted_coords, key=lambda coord: (coord[0], coord[1]))]

    # loop through the groups to find the difference in height
    for group in groupCoords:
        if len(group) < 2 or group[0] == group[1]:
            continue # jump over iteration if there is only one coordinate or lidar registered the same point twice

        # find max and min height
        min_height = min(coords[2] for coords in group)
        max_height = max(coords[2] for coords in group)

        # difference between them
        difference = max_height - min_height
        height_differences.append(difference)

    # list of thickness in a area
    return height_differences

# find the height of an area based on the coordinates of it's center
# and it's affiliations (subId and groupId) (soon to be implemented
def height_in_area(center):
    # container for all the heights in area
    area_heights = []

    # Limit the data to specific areas

    # this is only temporary data for coordinates in arctic
    # areas = [
    #     [(-3671212, 7898422), (-3200175, 4699978)],
    #     [(60.815356, 10.672022), (60.774878, 10.768867)],
    # ]

    # specified the area and it's range
    area = calculate_corners(center, 1500)
    area = (area[0], area[2])
    # NB: is only for the test data (soon to be removed)
    areazone = [(((area[0]) * ((-3671212/60.815356) - (-3200175/60.774878))) - 3200175, (area[1] * ((7898422/10.768867) - (4699978/10.672022))) + 4699978),
                (((area[0] - 100) * ((-3671212/60.815356) - (-3200175/60.774878))) - 3200175, ((area[1] - 1) * ((7898422/10.768867) - (4699978/10.672022))) + 4699978)]

    # Refactor lidar data to a readable format
    iceOver = laspy.read(lazData_path[0])
    iceUnder = laspy.read(lazData_path[1])

    # add two files together temporary test data(soon to be removed)
    ice_points = list(zip(iceOver.X,iceOver.Y,iceOver.Z)) + list(zip(iceUnder.X,iceUnder.Y,iceUnder.Z))

    # area height in all the subsections
    grid_area_heights = define_gridareas(areazone, 1500, 20)

    # find the heights of each sub-area => area-heights
    for sub_area in grid_area_heights:
        ice_points = list(filter(lambda point_position: inArea(point_position, (sub_area[0],sub_area[2])), ice_points))
        area_heights.append(find_height(ice_points))

    return area_heights
