import geopandas as gpd
from shapely.geometry import Polygon, Point, LineString
import matplotlib.pyplot as plt
import numpy as np


def get_relation(self, body_of_water: str):
    print("In get_relation")
    # Load GeoJSON data using geopandas
    geo_data = gpd.read_file("server/map/mjosa.geojson")

    # Filter only polygons, exclude points and other feature types to reduce response size
    polygon_data = geo_data[geo_data['geometry'].geom_type == 'Polygon']

    # Extract coordinates from polygons and create polygon objects
    polygons = [Polygon(polygon.exterior) for polygon in polygon_data['geometry']]

    if len(polygons) <= 1:  # Return if conversion to polygon fails
        print("Failed to convert to polygons")
        return

    print("Performing div call")
    tiles = divide_relation(polygons, 0.01)  # NB: any size under 0.02 will require patience
    print("Executed div call")

    # NB: temp test plot of map
    tiles.plot(color='blue', edgecolor='orange', linewidth=0.5)
    plt.show()

    # Convert GeoDataFrame to GeoJSON
    tiles_json = tiles.to_json()

    # Set headers
    self.send_response(200)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write GeoJSON to response object
    self.wfile.write(tiles_json.encode('utf-8'))


def divide_relation(polygons, cell_size):
    # Calculate the combined bounding box of all polygons
    combined_bounds = polygons[0].bounds
    for polygon in polygons[1:]:
        combined_bounds = (min(combined_bounds[0], polygon.bounds[0]),
                           min(combined_bounds[1], polygon.bounds[1]),
                           max(combined_bounds[2], polygon.bounds[2]),
                           max(combined_bounds[3], polygon.bounds[3]))

    # Create an empty list to store polygons representing tiles
    tiles = []

    # Iterate over each polygon
    for polygon in polygons:
        # Iterate over each cell
        for i in np.arange(combined_bounds[1], combined_bounds[3], cell_size):
            for j in np.arange(combined_bounds[0], combined_bounds[2], cell_size):
                # Define the bounds of the cell
                cell_bounds = (j, i, j + cell_size, i + cell_size)

                # Create a polygon representing the cell
                cell_polygon = Polygon([(cell_bounds[0], cell_bounds[1]),
                                        (cell_bounds[2], cell_bounds[1]),
                                        (cell_bounds[2], cell_bounds[3]),
                                        (cell_bounds[0], cell_bounds[3])])

                # Check if the cell polygon intersects with the current polygon
                if polygon.intersects(cell_polygon):
                    tiles.append(cell_polygon)

    # Create a GeoDataFrame from the list of tiles
    tiles_gdf = gpd.GeoDataFrame(geometry=tiles)

    return tiles_gdf


def divide_relation_2(polygons, cell_size):
    polygon = Point(0, 0).buffer(2).difference(Point(0, 0).buffer(1))
    line1 = LineString([(0, 0), (3, 3)])
    line2 = LineString([(0, 0), (3, -3)])

    line1_pol = line1.buffer(1e-3)
    line2_pol = line2.buffer(1e-3)

    new_polygon = polygon.difference(line1_pol).difference(line2_pol)
